# This script is used to set the OctopusMachineId  tags on the EC2 instance
# This tag is used by cloudwatch to auto deregister the tentacle when the instance is unavailable

Start-Transcript -path "D:\AddOctopusMachineIdTagLog.txt" -append

Add-Type -Path "C:\Program Files\Octopus Deploy\Tentacle\Newtonsoft.Json.dll" # Path to Newtonsoft.Json.dll 
Add-Type -Path "C:\Program Files\Octopus Deploy\Tentacle\Octopus.Client.dll" # Path to Octopus.Client.dll 

$availabilityZone = Invoke-WebRequest http://169.254.169.254/latest/meta-data/placement/availability-zone -UseBasicParsing 
$Region = $availabilityZone.Content.Trim("a","b","c")

$OctopusURL = "http://YOUR_OCTOPUS_IP_AND_PORT_HERE/" 
$OctopusAPIKey = "YOUR_OCTOPUS_API_KEY_HERE" 

$MachineName = (Invoke-RestMethod -Method Get -Uri http://169.254.169.254/latest/meta-data/instance-id)

$endpoint = new-object Octopus.Client.OctopusServerEndpoint $OctopusURL,$OctopusAPIKey 
$repository = new-object Octopus.Client.OctopusRepository $endpoint 
$findmachine = $repository.Machines.FindByName("$MachineName") 
$machineid = $findmachine.id

# Set the OctopusMachineId tag
New-EC2Tag `
        -Resource $MachineName `
        -Tag @{ Key="OctopusMachineId"; Value=$machineid } `
        -Region $Region `

Write-Output "Set OctopusMachineId to $machineid"

# Write any logs to S3 at the end of the script generating them.
Write-S3Object -bucketname pelican-deploy -File D:\AddOctopusMachineIdTagLog.txt -key ServerSetupLogs/$instanceId/AddOctopusMachineIdTagLog.txt

