# This script tells the Octopus server to deploy any releases this 
# instance should be running.  It looks at the Environment and roles
# to determine what release # should be deployed, then sends commands 
# to the server to begin that deployment.
# This is run only on the initial startup, and is launched by ServerSetup.ps1

# Variables
$apiKey = "YOUR_OCTOPUS_API_KEY_HERE" #Octopus API Key
$OctopusURL = "http://YOUR_OCTOPUS_IP_AND_PORT_HERE" #Octopus URL

# Start Logging
Start-Transcript -path "D:\AutoDeploy.txt" -append

# Load the cmdlet to get the current instance tags
Import-Module .\MyEC2Instance
 
$Header =  @{ "X-Octopus-ApiKey" = $apiKey } # This header is used in all the octopus rest api calls

$EnvironmentName = Get-MyTag -name Environment

# Get the project names and replace any periods with - for the rest calls (periods in a URI are of course problematic)
$ProjectNames = Get-MyTag -name Roles
$ProjectNames  = $ProjectNames.replace(".", "-")
 
# Function to request the release for a project
# This is called once for each project/role this EC2 instance is a member of
function GetReleaseForProject
{
	param (
		[Parameter(Mandatory=$True)]
		[string]$projectName
	)
	
	Write-Output "Getting Build for $projectName"
  
	# Get our Octopus Machine Id
	# This is needed for our rest calls and is the internal machine ID Octopus uses
	# We get all machines here, then find the one who's name matches our instanceId, then select the Id to use later
	$instanceId = (Invoke-RestMethod -Method Get -Uri http://169.254.169.254/latest/meta-data/instance-id)
	$allMachines = Invoke-WebRequest -UseBasicParsing http://YOUR_OCTOPUS_IP_AND_PORT_HERE/api/machines -header $Header | ConvertFrom-Json
	$OctopusMachineId =  $allMachines.Items.where({$_.Name -eq "$instanceId"}).Id
	
	# Getting Environment and Project By Name
	$fullUri = "$OctopusURL/api/projects/$ProjectName"
	$Project = Invoke-WebRequest -UseBasicParsing  -Uri $fullUri -Headers $Header| ConvertFrom-Json
	$Environment = Invoke-WebRequest -UseBasicParsing  -Uri "$OctopusURL/api/Environments/all" -Headers $Header| ConvertFrom-Json
	$Environment = $Environment | ?{$_.name -eq $EnvironmentName}
	
	# Finally set the environment and project id strings
	$environmentId = $Environment.Id
	$projectId = $Project.Id
	
	# Get the most recent release that matches our environmentId & projectId
	$fullUri = "$OctopusURL/api/deployments?Environments=$environmentId&Projects=$projectId&SpecificMachineIds=instanceId&Take=1"
	$currentRelease =  Invoke-WebRequest -UseBasicParsing  -Uri "$fullUri"  -Headers $Header  | ConvertFrom-Json

	# Set our machine name in an array of MachineNames to be converted into a JSON array for the rest call
	[string[]] $MachineNames = $OctopusMachineId
	
	# Generate the JSON for our rest call by creating an object in powershell and using ConvertTo-Json
	$DeploymentBody = @{ 
				ReleaseID = $currentRelease.Items[0].ReleaseID
				EnvironmentID = $Environment.id
				SpecificMachineIds = $MachineNames
			  } | ConvertTo-Json
			  
	$fullUri =  "$OctopusURL/api/deployments"

	# Make the rest call to start a deployment
	$deploymentCall = Invoke-WebRequest -UseBasicParsing  -Uri $fullUri  -Method Post -Headers $Header -Body $DeploymentBody
}


# Split all of our project names into an array to loop over
$projectsSplit = $ProjectNames.split("|")
Write-Output "Split Projects $projectsSplit"

# Call GetReleaseForProject for each role/project this instance is a member of
foreach($projectName in $projectsSplit)
{
	GetReleaseForProject $projectName
}

# Write the auto deploy log to S3
Write-S3Object -bucketname pelican-deploy -File D:\AutoDeploy.txt -key ServerSetupLogs/$instanceId/AutoDeploy.txt