# This script is used to install required HTTPS certificates on the EC2 instance
# It looks at the roles the instance is in, then downloads and installs any required certificates from S3
# ex. Production www.rapsio.com  
#     Development www.pubsio.com

Start-Transcript -path "D:\InstallCertificates.txt" -append

# Import any custom modules here
Import-Module AWSPowerShell
Import-Module .\MyEC2Instance

# Get Variables we need

$instanceId = (Invoke-RestMethod -Method Get -Uri http://169.254.169.254/latest/meta-data/instance-id)
$sourceBucketName = "YOUR_S3_BUCKET_NAME_HERE"
$registerInRoles = Get-MyTag -name Roles
$instanceName = "$registerInEnvironments $registerInRoles"
$localPath = "c:\temp"

set-location cert:\localMachine\my


function Import-PfxCertificate 
{
	param([String]$certPath,[String]$certRootStore = "LocalMachine",[String]$certStore = "My")
	
	$pfxPass = "cc540540!" | ConvertTo-SecureString -AsPlainText -Force
	$pfx = new-object System.Security.Cryptography.X509Certificates.X509Certificate2
	
	$pfx.import($certPath,$pfxPass,"PersistKeySet")
	$store = new-object System.Security.Cryptography.X509Certificates.X509Store($certStore,$certRootStore)
	$store.open("MaxAllowed")
	$store.add($pfx)
	Write-Output "Store $certStore RootStore $certRootStore"
	$store.close()
}

function Download-Certificates()
{
	param 
	(
		[Parameter(Mandatory=$True)]
		[string]$role
	)
	
	$keyPrefix = "Certificates/$role"

	$objects = get-s3object -bucketname $sourceBucketName -KeyPrefix $keyPrefix
	foreach($object in $objects) 
	{
		$localFileName = $object.Key -replace $keyPrefix, ''
		Write-Output "Copying File to $localFileName"
		if ($localFileName -ne '' -and $localFileName -ne '/') 
		{
			$localFilePath = Join-Path $localPath $localFileName
			Write-Output "Copying File " $localFileName " to " $localFilePath
			Copy-S3Object -BucketName $sourceBucketName -Key $object.Key -LocalFile $localFilePath
			Write-Output "Installing Certificate $localFilePath"
			Import-PfxCertificate  $localFilePath
			Write-Output "Certificate Install Complete"
		}
		
	}
}

foreach($roleName in $registerInRoles.Split("{|}"))
{
	Download-Certificates $roleName
}


# Write any logs to S3 at the end of the script generating them.
Write-S3Object -bucketname pelican-deploy -File D:\InstallCertificates.txt -key ServerSetupLogs/$instanceId/InstallCertificates.txt

