# This script is used to set custom tags on the EC2 instance
# It currently only sets the name tag to a combination of the Environment & Roles tags
# ex. Production www.rapsio.com  
#     Development www.pubsio.com

Start-Transcript -path "D:\SetTagsLog.txt" -append

# Import any custom modules here
Import-Module AWSPowerShell
Import-Module .\MyEC2Instance

# Get Variables we need
$availabilityZone = Invoke-WebRequest http://169.254.169.254/latest/meta-data/placement/availability-zone -UseBasicParsing 
$Region = $availabilityZone.Content.Trim("a","b","c")
$registerInEnvironments = Get-MyTag -name Environment
$registerInRoles = Get-MyTag -name Roles

$instanceName = "$registerInEnvironments $registerInRoles"

# Sets the name tag to $instanceName
function Set-InstanceNameTag()
{
    $instanceId = (Invoke-RestMethod -Method Get -Uri http://169.254.169.254/latest/meta-data/instance-id)

    #Get the current value of the name tag
   	
	Remove-EC2Tag `
        -Resource $instanceId `
        -Tag @{ Key="Name" } `
        -Region $Region `
        -ProfileName $ProfileName `
        -Force
		
	Write-Output "Setting Name to $instanceName"
		
	New-EC2Tag `
        -Resource $instanceId `
        -Tag @{ Key="Name"; Value=$instanceName } `
        -Region $Region `
}

Set-InstanceNameTag

# Write any logs to S3 at the end of the script generating them.
Write-S3Object -bucketname pelican-deploy -File D:\SetTagsLog.txt -key ServerSetupLogs/$instanceId/SetTagsLog.txt

